/*
 * This program was written by Grzegorz Adam Hankiewicz and Carlos Pereda
 * Moreno and is Giftware: you are free to do what you want with it without
 * any restriction. We do not accept responsibility for any effects, adverse
 * or otherwise, that this program may have on you, your computer, your
 * sanity, your dog, and anything else that you can think of. Use it at your
 * own risk.
*/
import java.awt.Graphics;

/** Esta es la clase base que compone el array del mapa. Contiene informacin
	* privada sobre grfico cargado, si la casilla puede ser cruzada por un coche,
	* si la casilla necesita ser redibujada externamente (til para no redibujar
	* todo el mapa y evitar parpadeos), si hay una explosin en la casilla, y toda
	* la informacin heredada de la clase Flecha.
	* 
	* Aparte de tener varios mtodos simples para leer/escribir los valores de las
	* variables privadas, hay dos mtodos (set_tile_suelo y dibuja_casilla) que
	* conviene detallar: el primero recibe los parmetros de las casillas
	* colindantes durante la carga del mapa lgico de un fichero, y en base a eso
	* se autodesigna un tile grfico que represente la casilla.
	* 
	* El mtodo dibuja_casilla es llamado desde fuera por cada casilla, determina si es
	* necesario dibujar una casilla, y luego dibuja el suelo, los coches, las
	* flechas y las explosiones en ese orden si las hubiese.
	*/

class Casilla extends Flecha
{
	int logico; // Valor lgico del mapa (0 pared, 1 carretera)
	int tile_suelo; // Nmero del tile grfico que representa ese tile
	int coche; // Indice al array de coches si hay un coche, o -1
	boolean dirty; // Indica si se debe redibujar la casilla.
	boolean explosion;

	Casilla ()
	{
		coche = -1;
		dirty = true;
	}

	void set_dirty (boolean valor) { dirty = valor; }
	boolean get_dirty () { return dirty; }
	
	public void set_coche (int val) { coche = val; dirty = true; }
	public int get_coche () { return coche; }
	public void set_logico (int val) { logico = val; }
	public int get_logico () { return logico; }
	public int get_tile_suelo () { return tile_suelo; }

	/** Funcin que establece el valor del tile dependiendo de los valores
	   * lgicos up, down, left, right, que son los de las casillas
		* colindantes.
		*/
	public void set_tile_suelo (int up, int down, int left, int right, int centro)
	{
		int val = up + down * 2 + right * 4 + left * 8;

		if (centro > 0) val = 15;
		tile_suelo = val;

		if (val != 3 && val != 12 && val < 15)
			set_flecha (up*(-1) + 1, down*(-1) + 1, left*(-1) + 1, right*(-1) + 1);

		/* Parche para variar los grficos de fondo */
		if (val == 15)
		{
			int num = (int) (Math.random() * 6);
			if (num > 0)
				tile_suelo = 25 + num;
		}
	}

	public void explota ()
	{
		explosion = true;
		set_dirty (true);
	}

	public void borra_explosion()
	{
		explosion = false;
		set_dirty (true);
	}

	public void dibuja_casilla (Graphics g, TileSet tiles, Coche coches[], int pos_x, int pos_y, java.awt.image.ImageObserver extra)
	{
		if (!dirty)
			return ;
			
		// Dibujamos el tile del suelo
		g.drawImage (tiles.imagenes[get_tile_suelo()], pos_x, pos_y, extra);

		int c = get_coche();
		if (c >= 0)
		{
			if (coches[c] != null)
				g.drawImage (tiles.imagenes[15 + coches[c].get_dir()], pos_x, pos_y, extra);
		}

		if (se_puede_girar_flecha())
			g.drawImage (tiles.imagenes[get_tile_flecha()], pos_x, pos_y, extra);

		dirty = false;

		if (explosion)
		{
			g.drawImage (tiles.imagenes[24], pos_x, pos_y, extra);
			borra_explosion();
		}
	}
}
