/*
 * This program was written by Grzegorz Adam Hankiewicz and Carlos Pereda
 * Moreno and is Giftware: you are free to do what you want with it without
 * any restriction. We do not accept responsibility for any effects, adverse
 * or otherwise, that this program may have on you, your computer, your
 * sanity, your dog, and anything else that you can think of. Use it at your
 * own risk.
*/
/** Esta es una clase virtual, que necesariamente ser ser extendida por otra
  * clase. Implementa varios mtodos lgicos para establecer flechas y sus
  * direcciones, dos mtodos para girar la flecha tanto a la izquierda como
  * a la derecha, y llama a algunos mtodos virtuales que deben ser implementados
  * por las clases hijas, que son necesarios para redibujar las casillas y por
  * eso no se incluyen en esta clase, ya que esta clase no tiene mtodo de
  * dibujado.
  * 
  * Cabe destacar que se han usado flechas "invisibles" para determinar ciertas
  * direcciones de los coches. Por ejemplo, todas las curvas son flechas
  * bidireccionales que apuntan en ambas direcciones. Un coche determina el
  * giro que debe tomar dependiendo de su propia direccin. Los callejones
  * sin salida son tambin implementados como flechas invisibles, que no se
  * pueden girar y siempre apuntan en la direccin contraria.
  */
abstract class Flecha
{
	private int flecha; // -1 si no hay flecha
	private boolean giro[];

	private final static int NUM_GIROS = 4;

	public final static int FAR = 0;
	public final static int FDE = 1;
	public final static int FAB = 2;
	public final static int FIZ = 3;
	public final static int FARI = 5;
	public final static int FARD = 9;
	public final static int FABI = 6;
	public final static int FABD = 10;

	private void corrige_desborde_flecha()
	{
		if (flecha < 0) flecha = NUM_GIROS - 1;
		else if (flecha >= NUM_GIROS) flecha = 0;
	}

	Flecha ()
	{
		flecha = -1;
		giro = new boolean[NUM_GIROS];
	}

	public boolean se_puede_girar_flecha()
	{
		int num_giros = 0;

		for (int x = 0; x < NUM_GIROS; x++)
			if (giro[x]) num_giros++;

		if (num_giros >= 3)
			return true;
		
		return false;
	}

	public void set_flecha (int up, int down, int left, int right)
	{
		if (up != 0) giro[FAR] = true;
		if (down != 0) giro[FAB] = true;
		if (left != 0) giro[FIZ] = true;
		if (right != 0) giro[FDE] = true;

		int suma = up + down + left + right;
		if (suma == 1)
		{
			flecha = up * FAR + down * FAB + left * FIZ + right * FDE;
		}
		else if (suma == 2)
		{
			flecha = up * 1 + down * 2 + left * 4 + right * 8;
		}
		else
		{
			flecha = (int) (Math.random() * 3.9);
			if (!giro[flecha])
				flecha++;

			corrige_desborde_flecha();
		}

		set_dirty (true);
	}

	public int get_tile_flecha()
	{
		if (flecha == FAR || flecha == FIZ || flecha == FDE || flecha == FAB)
			return 20 + flecha;

		return -1;
	}

	public void rota_flecha_izquierda ()
	{
		if (!se_puede_girar_flecha())
			return ;

		flecha--;
		corrige_desborde_flecha();

		if (!giro[flecha])
		{
			flecha--;
			corrige_desborde_flecha();
		}

		set_dirty (true);
	}

	public void rota_flecha_derecha ()
	{
		if (!se_puede_girar_flecha())
			return ;

		flecha ++;
		corrige_desborde_flecha();
		
		if (!giro[flecha])
		{
			flecha++;
			corrige_desborde_flecha();
		}

		set_dirty (true);
	}

	public int get_flecha () { return flecha; }
	public boolean hay_flecha () { if (flecha >= 0) return true; return false;}
	
	abstract void set_dirty (boolean valor);
	abstract boolean get_dirty ();
}
